/*
   File: ast_utils.c
   Defines some support routines to handle the ast

   Copyright (C) 2008 Marc Seutter

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   CVS ID: "$Id: ast_utils.c,v 1.8 2011/08/13 15:57:30 marcs Exp $"
*/

/* System includes */
#include <stdio.h>
#include <stdlib.h>

/* libdcg includes */
#include <dcg.h>
#include <dcg_error.h>

/* Local includes */
#include "cdl_ds.h"
#include "affix_rules.h"

string string_from_guard_type (guard_type type)
{ switch (type)
    { case undetermined:	return ("undetermined");
      case equal:		return ("equal");
      case split:		return ("split");
      case join:		return ("join");
      case assign:		return ("assign");
      case arith_join:		return ("arithmetic join");
      case arith_split:		return ("arithmetic split");
      default: dcg_bad_tag (type, "string_from_guard_type");
    };
  return (NULL);
}

string string_from_operator (operator op)
{ switch (op)
    { case times:		return ("*");
      case modulo:		return ("%%");
      case divides:		return ("%");
      case plus:		return ("+");
      case minus:		return ("-");
      case shift_left:		return ("<<");
      case shift_right:		return (">>");
      case bitwise_and:		return ("&");
      case bitwise_or:		return ("||");
      case bitwise_xor:		return ("^");
      case bitwise_not:		return ("~");
      /* identified operators */
      case int_times_int:	return ("*");
      case real_times_real:	return ("*");
      case int_times_text:	return ("*");
      case int_plus_int:	return ("+");
      case real_plus_real:	return ("+");
      case text_plus_text:	return ("+");
      case int_minus_int:	return ("-");
      case real_minus_real:	return ("-");
      default: dcg_bad_tag (op, "string_from_operator");
    };
  return ("");
}

string string_from_proc_type (proc_type ptype)
{ switch (ptype)
    { case prelude:		return ("PRELUDE");
      case interlude:		return ("INTERLUDE");
      case postlude:		return ("POSTLUDE");
      case function:		return ("FUNCTION");
      case test:		return ("TEST");
      case action:		return ("ACTION");
      case predicate:		return ("PREDICATE");
      default:			dcg_bad_tag (ptype, "string_from_proc_type");
    };
  return ("");
}

affix_rule type_from_operator (operator op)
{ switch (op)
    { case times:		break;
      case modulo:		return (get_primitive_type (prim_int));
      case divides:		return (get_primitive_type (prim_int));
      case plus:		break;
      case minus:		break;
      case shift_left:		return (get_primitive_type (prim_int));
      case shift_right:		return (get_primitive_type (prim_int));
      case bitwise_and:		return (get_primitive_type (prim_int));
      case bitwise_or:		return (get_primitive_type (prim_int));
      case bitwise_xor:		return (get_primitive_type (prim_int));
      case bitwise_not:		return (get_primitive_type (prim_int));
      /* identified operators */
      case int_times_int:	return (get_primitive_type (prim_int));
      case real_times_real:	return (get_primitive_type (prim_real));
      case int_times_text:	return (get_primitive_type (prim_text));
      case int_plus_int:	return (get_primitive_type (prim_int));
      case real_plus_real:	return (get_primitive_type (prim_real));
      case text_plus_text:	return (get_primitive_type (prim_text));
      case int_minus_int:	return (get_primitive_type (prim_int));
      case real_minus_real:	return (get_primitive_type (prim_real));
      default: dcg_bad_tag (op, "type_from_operator");
    };
  return (AFFIX_ERROR);
}

void set_local_state (state_list sl, int nr, state ns)
{ /* Make sure that the index exists */
  room_state_list (sl, nr + 1);
  if (sl -> size <= nr + 1) sl -> size = nr + 1;
  sl -> array[nr] = ns;
}

state get_local_state (state_list sl, int nr)
{ /* Note that the state_list may not contain this index, in which case it must be undefined */
  if (nr >= sl -> size) return (undefined);
  return (sl -> array[nr]);
}
