/*
   File: imc_utils.h
   Defines support routines to handle the intermediate code

   Copyright (C) 2008 Marc Seutter

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   CVS ID: "$Id: imc_utils.h,v 1.6 2010/08/04 14:00:38 marcs Exp $"
*/
#ifndef IncImcUtils
#define IncImcUtils

/* libdcg includes */
#include <dcg.h>

/* local includes */
#include "cdl_ds.h"

/* Block manipulation */
void append_instr (block bb, instr ins);
void prepend_instr (block bb, instr ins);
void remove_block_from_list (block_list bl, block bb);
void replace_last_instruction (block bb, instr newlast);

/* Label administration */
block get_block_from_label (int label);
int pick_or_create_new_block_label (block bb);

/* exported code */
state get_variable_state (block bb, address var_adr);
void set_variable_state (block bb, address var_adr, state s);
void update_variable (block bb, address var_adr, instr loc, state ns);

/* Instruction manipulation */
int  instr_has_fail_label (instr ins);
void set_fail_label (instr ins, int label);
int pick_fail_label (instr ins);
int is_a_local_address (address adr, int *lnr);
int is_a_reference (address adr, address *deref);
int is_a_local_detach (instr ins, int *lnr);
int is_a_local_attach (instr ins, int *lnr);
int is_a_call (instr ins, address_list *args);
int is_a_local_update (instr ins, int *lnr);
int is_a_local_use (instr ins, int lnr);
void calc_local_gen_use (instr ins, char_list gen, char_list use);

/* Intermediate code generation support */
void connect_true_successor (block *current, proc pdef);
void connect_fail_successor (block fail_bb, block follower);
void connect_new_fail_successor (block fail_bb, block prev_rec_bb, proc pdef,
                                 int *fail_label, block *current);

/* Debug and Init */
void dump_proc_name (proc pdef);
void dump_proc (proc pdef);
void dump_intermediate_code ();
void init_imc_utils ();

#endif /* IncImcUtils */

