/*
   File: incr_error.c
   Adds support to enter an editor interactively upon errors

   Copyright (C) 2010 Marc Seutter

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   CVS ID: "$Id: incr_error.c,v 1.1 2010/02/10 15:09:00 marcs Exp $"
*/

/* System includes */
#include <stdio.h>
#include <stdlib.h>

/* libdcg includes */
#include <dcg.h>

/* Local includes */
#include "options.h"

static void edit_source (char *fname, int line, int col)
{ char *editor = getenv ("EDITOR");
  char cmd[MAXSTRLEN];
  if (editor == NULL) editor = "vi";
  sprintf (cmd, "%s +%d %s", editor, line, fname);
  fprintf (stderr, "collected command: '%s'\n", cmd);
  system (cmd);
}

static void enter_editor (char *fname, int line, int col)
{ char buf[MAXSTRLEN];
  fprintf (stdout, "Do you want to enter the editor (y/n)? ");
  fflush (stdout);
  if (fgets (buf, MAXSTRLEN - 1, stdin) == NULL) return;
  if (buf[0] == 'y') edit_source (fname, line, col);
}

void maybe_enter_editor (char *fname, int line, int col)
{ char *env_value = getenv ("CDLINTERACTIVE");
  if ((env_value != NULL) || interactive_compilation)
    { enter_editor (fname, line, col);
      exit (4);
    };
}

