/*
   File: lexer.h
   Defines the lexical analyzer for CDL3

   Copyright (C) 2008 Marc Seutter

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   CVS ID: "$Id: lexer.h,v 1.3 2008/06/28 13:38:29 marcs Exp $"
*/
#ifndef IncLexer
#define IncLexer

/* libdcg includes */
#include <dcg.h>

/* local includes */
#include "cdl_ds.h"

/* exported actions */
void should_be_symbol (string sy);
void should_be_module_name (string *mname);
void should_be_affix_name (string *aname);
void should_be_affix_variable_name (string *aname);
void should_be_text_constant (string *text);
int ahead_symbol (string sy);
int is_keyword (string key);
int is_symbol (string sy);
int is_module_name (string *mname);
int is_name_part (string *part);
int is_affix_name (string *aname);
int is_affix_terminal (string *aname);
int is_affix_variable_name (string *aname);
int is_number (int *kind, int *inum, real *rnum);
int is_text_constant (string *text);

/*
  Lexer position interface
*/ 
void get_erroneous_name (string *rname);
void get_current_filename (string *rfname);
void get_current_position (int *rline, int *rcol);
int end_of_file ();
void init_lexer (FILE *fd, char *fname);
void finish_lexer ();

#endif /* IncLexer */
