/*
   File: proc_tree.c
   Handles trees of declarations

   Copyright (C) 2008 Marc Seutter

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   CVS ID: "$Id: proc_tree.c,v 1.8 2011/09/05 19:56:22 marcs Exp $"
*/

/* standard includes */
#include <stdio.h>
#include <string.h>

/* support lib includes */
#include <dcg.h>
#include <dcg_alloc.h>
#include <dcg_string.h>
#include <dcg_dump.h>

/* local includes */
#include "cdl_ds.h"
#include "proc_tree.h"

static void detach_signatures (pdecl_list pdefs)
{ int ix;
  if (pdefs == pdecl_list_nil) return;
  for (ix = 0; ix < pdefs -> size; ix++)
    { nonrec_detach_psig_list (&pdefs -> array[ix] -> p1sig);
      nonrec_detach_psig_list (&pdefs -> array[ix] -> p2sig);
    };
}

/* introduce necessary routines and shorthands */
void detach_proc_tree (proc_tree *optr)
{ proc_tree old = (proc_tree) dcg_predetach ((void **) optr);
  if (old == proc_tree_nil) return;
  detach_string_list (&old -> tag);
  detach_signatures (old -> pdefs);	/* Special */
  detach_pdecl_list (&old -> pdefs);
  detach_proc_tree (&old -> left);
  detach_proc_tree (&old -> right);
  dcg_detach ((void **) &old);
};

/* supplementary routines and shorthands */
static void my_ppp_proc_tree (FILE *f, int ind, proc_tree old)
{ if (old == proc_tree_nil) return;
  my_ppp_proc_tree (f, ind, old -> left);
  pppdelim (f, 0, ind, '<');
  ppp_string_list (f, 1, ind + 2, old -> tag);
  pppdelim (f, 0, ind, ',');
  ppp_pdecl_list (f, 0, ind + 2, old -> pdefs);
  pppdelim (f, 0, ind, '>');
  my_ppp_proc_tree (f, ind, old -> right);
};

void ppp_proc_tree (FILE *f, int horiz, int ind, proc_tree old)
{ pppdelim (f, 0, ind, '{');
  my_ppp_proc_tree (f, ind + 2, old);
  pppdelim (f, 0, ind, '}');
};

void save_proc_tree (FILE *f, proc_tree old)
{
};

int load_proc_tree (FILE *f, proc_tree *x)
{ return (0);
};

/*
   Specific routines and shorthands:
   We can't do much semantics here, compared to affix_tree,
   because the rule check now also has to resolve the overloading
   (I would not like overloading on the affix level ;-)
*/
pdecl_list enter_proc_tree (proc_tree *root, string_list pname_parts)
{ proc_tree *ptr = root;
  proc_tree new;
  while (*ptr != proc_tree_nil)
    { proc_tree current = *ptr;
      int cond = cmp_string_list (pname_parts, current -> tag);
      if (cond < 0) ptr = &current -> left;
      else if (cond > 0) ptr = &current -> right;
      else /* Already present in tree: return list of defs */
        return (current -> pdefs);
    };

  new = (proc_tree) dcg_malloc (sizeof (struct proc_tree_rec));
  new -> tag = attach_string_list (pname_parts);
  new -> pdefs = new_pdecl_list ();
  new -> left = proc_tree_nil;
  new -> right = proc_tree_nil;
  *ptr = new;
  return (new -> pdefs);
}

pdecl_list lookup_proc_tree (proc_tree root, string_list pname_parts)
{ proc_tree ptr = root;
  while (ptr != proc_tree_nil)
    { int cond = cmp_string_list (pname_parts, ptr -> tag);
      if (cond < 0) ptr = ptr -> left;
      else if (cond > 0) ptr = ptr -> right;
      else /* Found proc rule */ 
        return (ptr -> pdefs);
    };
  return (pdecl_list_nil);
}
