/*
   File: mrts_predef.h
   Defines the set of predefined operations for MCDL

   Copyright (C) 2008 Marc Seutter

   This library is free software: you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published   
   by the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU Lesser General Public License
   along with this library.  If not, see <http://www.gnu.org/licenses/>.

   CVS ID: "$Id: mrts_predef.h,v 1.15 2009/10/07 14:53:12 marcs Exp $"
*/
#ifndef IncMrtsPredef
#define IncMrtsPredef

/* For compiling with C++ */
#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/* Exported predefined macros: */
#define mrts_less_INT_INT(v1,v2)      (Value_INT (v1) <  Value_INT (v2))
#define mrts_lesseq_INT_INT(v1,v2)    (Value_INT (v1) <= Value_INT (v2))
#define mrts_greater_INT_INT(v1,v2)   (Value_INT (v1) >  Value_INT (v2))
#define mrts_greatereq_INT_INT(v1,v2) (Value_INT (v1) >= Value_INT (v2))

#define mrts_less_REAL_REAL(v1,v2)      (Value_REAL (v1) <  Value_REAL (v2))
#define mrts_lesseq_REAL_REAL(v1,v2)    (Value_REAL (v1) <= Value_REAL (v2))
#define mrts_greater_REAL_REAL(v1,v2)   (Value_REAL (v1) >  Value_REAL (v2))
#define mrts_greatereq_REAL_REAL(v1,v2) (Value_REAL (v1) >= Value_REAL (v2))

/* Exported predefined operations: */

/* File operations */
int mrts_open_input_file_TEXT_FILE  (value text, value *ret_file);
int mrts_open_output_file_TEXT_FILE (value text, value *ret_file);
int mrts_open_standard_output_FILE (value *ret_file);
int mrts_open_standard_error_FILE  (value *ret_file);
int mrts_open_standard_input_FILE  (value *ret_file);
int mrts_close_FILE (value file);
int mrts_seek_FILE_INT (value file, value offset);

/* Pipe operations */
int mrts_pipe_to_command_TEXT_FILE   (value text, value *ret_file);
int mrts_pipe_from_command_TEXT_FILE (value text, value *ret_file);
int mrts_close_pipe_FILE (value file);

/* Primitive write functions */
int mrts_write_TEXT (value text);
int mrts_write_INT  (value val);
int mrts_write_REAL (value val);
int mrts_write_char_INT  (value val);
int mrts_write_FILE_TEXT (value file, value text);
int mrts_write_FILE_INT  (value file, value val);
int mrts_write_FILE_REAL (value file, value val);
int mrts_write_char_FILE_INT (value file, value val);
int mrts_formatted_write_FILE_FORMAT (value file, value format);

/* Trace functions */
int mrts_trace_TEXT (value text);
int mrts_trace_INT  (value val);
int mrts_trace_REAL (value val);
int mrts_formatted_trace_FILE_FORMAT (value file, value format);

/* Primitive read functions */
int mrts_read_line_TEXT (value *ret_text);
int mrts_read_char_TEXT (value *ret_text);
int mrts_read_char_INT  (value *ret_int);
int mrts_read_line_FILE_TEXT (value file, value *ret_text);
int mrts_read_char_FILE_TEXT (value file, value *ret_text);
int mrts_read_char_FILE_INT  (value file, value *ret_int);

/* Flushing functions */
int mrts_flush (void);
int mrts_flush_FILE (value file);

/* Text arithmetic functions */
int mrts_head_TEXT_TEXT (value text, value *ret_text);
int mrts_tail_TEXT_TEXT (value text, value *ret_text);
int mrts_split_TEXT_TEXT_TEXT (value text, value *ret_text1, value *ret_text2);
int mrts_extract_TEXT_INT_INT_TEXT (value text, value lwb, value upb, value *ret_text);
int mrts_is_subtext_TEXT_INT_TEXT (value text1, value *trans_int, value text2);
int mrts_between_TEXT_INT_TEXT_TEXT (value text, value pos, value lwb, value upb);
int mrts_not_between_TEXT_INT_TEXT_TEXT (value text, value pos, value lwb, value upb);
int mrts_before_TEXT_TEXT (value text1, value text2);
int mrts_is_prefix_TEXT_TEXT_TEXT (value prefix, value text, value *ret_text);
int mrts_prefix_TEXT_INT_TEXT_TEXT (value text, value len, value *ret_text1, value *ret_text2);
int mrts_length_TEXT_INT (value text, value *ret_len);

/* Conversion operations */
int mrts_convert_to_INT_REAL (value ival, value *ret_real);
int mrts_round_to_REAL_INT (value rval, value *ret_int);
int mrts_trunc_to_REAL_INT (value rval, value *ret_int);
int mrts_convert_to_INT_TEXT (value ival, value *ret_text);
int mrts_convert_to_REAL_TEXT (value rval, value *ret_text);
int mrts_convert_to_TEXT_INT (value text, value *ret_int);
int mrts_convert_to_TEXT_REAL (value text, value *ret_real);

/* Legacy type conversion */
int mrts_asciicode_TEXT_INT_INT (value text, value ival, value *ret_int);
int mrts_bindec_INT_TEXT (value ival, value *ret_text);
int mrts_decbin_TEXT_INT (value text, value *ret_int);

/* System interface */
int mrts_command_arg_INT_TEXT (value nr, value *ret_text);
int mrts_exit_INT (value rval);
int mrts_get_from_environment_TEXT_TEXT (value text, value *ret_text);
int mrts_time_INT (value *ret_int);
int mrts_execute_TEXT (value text);
int mrts_file_date_TEXT_INT (value text, value *ret_int);

/* Miscellaneous */
int mrts_hash_TEXT_INT_INT (value text, value ival, value *ret_int);

/* Exported routines */
void mrts_init_predef (int argc, char **argv);

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* IncMrtsPredef */
